@@ -1,4 +1,4 @@
-#  Copyright (c) 2021 Mira Geoscience Ltd.
+#  Copyright (c) 2022 Mira Geoscience Ltd.
 #
 #  This file is part of geoh5py.
 #
@@ -44,7 +44,7 @@ class PropertyGroup(ABC):
         self._uid = uuid.uuid4()
         self._association: DataAssociationEnum = DataAssociationEnum.VERTEX
         self._properties: list[uuid.UUID] = []
-        self._property_group_type = "multi-element"
+        self._property_group_type = "Multi-element"
         self._parent = None
 
         for attr, item in kwargs.items():
@@ -63,16 +63,16 @@ class PropertyGroup(ABC):
         return self._association
 
     @association.setter
-    def association(self, value):
-        if self._association is None:
+    def association(self, value: str | DataAssociationEnum):
+        if isinstance(value, str):
+            value = getattr(DataAssociationEnum, value.upper())
 
-            if isinstance(value, str):
-                value = getattr(DataAssociationEnum, value.upper())
+        if not isinstance(value, DataAssociationEnum):
+            raise TypeError(
+                f"Association must be 'VERTEX', 'CELL' or class of type {DataAssociationEnum}"
+            )
 
-            assert isinstance(
-                value, DataAssociationEnum
-            ), f"Association must be of type {DataAssociationEnum}"
-            self._association = value
+        self._association = value
 
     @property
     def attribute_map(self) -> dict:
